package dk.contix.eclipse.hudson;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.Preferences;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class HudsonClient {
	
	private Preferences prefs;

	public HudsonClient() {
		prefs = Activator.getDefault().getPluginPreferences();
	}
	
	private String getBase() {
		return prefs.getString(Activator.PREF_BASE_URL);
	}

	public Job[] getJobs() throws IOException {
		try {
			URL u = new URL(getBase() + "/api/xml");
			InputStream is = u.openStream();
			Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(is);
			
			Element root = doc.getDocumentElement();
			NodeList jobNodes = root.getElementsByTagName("job");
			
			Job[] res = new Job[jobNodes.getLength()];
			for (int i = 0; i < res.length; i++) {
				Element jobNode = (Element) jobNodes.item(i);
				
				res[i] = new Job(getNodeValue(jobNode, "name"), getNodeValue(jobNode, "url"), getNodeValue(jobNode, "color"), getNodeValue(jobNode, "lastBuild"));
			}
			
			is.close();
			
			return res;
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (ParserConfigurationException e) {
			throw new RuntimeException(e);
		}
	}
	
	public void scheduleJob(String project) throws IOException {
		URL u = new URL(getBase() + "/job/" + project + "/build");
		HttpURLConnection connection = (HttpURLConnection) u.openConnection();
		connection.setAllowUserInteraction(false);
		connection.setConnectTimeout(1000);
		connection.setReadTimeout(1000);
		connection.setInstanceFollowRedirects(false);
		connection.getResponseCode();

		connection.disconnect();
	}
	
	public static void checkValidUrl(String base) throws Exception {
		URL u = new URL(base + "/api/xml");
		HttpURLConnection connection = (HttpURLConnection) u.openConnection();
		connection.setAllowUserInteraction(false);
		connection.setConnectTimeout(1000);
		connection.setReadTimeout(1000);
		connection.setInstanceFollowRedirects(false);
		connection.getResponseCode();

		Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(connection.getInputStream());

		Element root = doc.getDocumentElement();
		if (!root.getLocalName().equals("hudson") || !root.getFirstChild().getLocalName().equals("jobs")) {
			throw new IllegalArgumentException("URL does not point to a valid Hudson installation. /api/xml does not return correct data.");
		}
	}

	private String getNodeValue(Element node, String name) {
		NodeList list = node.getElementsByTagName(name);
		if (list.getLength() == 1) {
			return list.item(0).getTextContent().trim();
		}
		return null;
	}

}
